// interface definition for IConfiguration
//
// Copyright Matthias Gaertner, Schenefeld, Germany, 2001
// You may use this code under the terms of the GPL.
// Updated packages may be found at http://www.rtner.de/parsled/
//

#ifndef __CONFIGURATION_H
#define __CONFIGURATION_H

#include <stdio.h>
#include <IConfiguration.h>

//---

struct _ConfigurationListElement
{
    _ConfigurationListElement();
    ~_ConfigurationListElement();

    struct _ConfigurationListElement *_next;
    IConfigClass *_c;
    SimpleString *_name;
};

//---

class ConfigSourceFile : public IConfigurationSource
{
public:
    ConfigSourceFile( FILE *f );

    bool eof( void ) const;
    int nextc( void );

protected:
    FILE *_f;
};

//---

class ConfigSourceString : public IConfigurationSource
{
public:
    ConfigSourceString( const SimpleString *s );

    bool eof( void ) const;
    int nextc( void );

protected:
    const SimpleString *_s;
    unsigned int _n;
};

//---

class Configuration : public IConfiguration
{
public:
    Configuration();
    ~Configuration();

    void reset( void );
    int feedConfiguration( IConfigurationSource *ics );
    int feedConfiguration( const SimpleString *s );
    int feedConfiguration( FILE *f );

    IConfigClass *getSection( SimpleString *s ) const;
    void resetIter( void );
    void pushIter( void );  // extremely clumsy...
    void popIter( void );
    IConfigClass *getNextIter( void );


protected:
    virtual IConfigClass *createClassForSection( const SimpleString *section ) = 0;

    IConfigLine *createLine( const SimpleString *currentSection );
    void addSection( const SimpleString *section, IConfigClass *cl );

    struct _ConfigurationListElement *unlinkSectionByName( const SimpleString *section );
    struct _ConfigurationListElement *unlinkSectionByClass( const IConfigClass *cl );

    struct _ConfigurationListElement *_l;
    struct _ConfigurationListElement *_cursor;
    struct _ConfigurationListElement *_cursor2;

private:
    void _reset( struct _ConfigurationListElement *p );
};

#endif /* __CONFIGURATION_H */

/* */
