// Implementation SimpleString
//
// Copyright Matthias Gaertner, Schenefeld, Germany, 2001
// You may use this code under the terms of the GPL.
// Updated packages may be found at http://www.rtner.de/parsled/
//

#include <SimpleString.h>
#include <string.h>

//---- Constructors/Destrucor

void SimpleString::_init( const unsigned char *s, unsigned int length )
{
    if( s == NULL )
    {
        _n = 0;
        length = (length<=0) ? 10 : length;
        _nAllocated = length;
        _s = new unsigned char[_nAllocated];
        memset( (char*)_s, 0, _nAllocated );
    }
    else
    {
        _n = length;
        _nAllocated = length + 10;
        _s = new unsigned char[_nAllocated];
        memset( (char*)_s, 0, _nAllocated );
        if( length > 0 )
        {
            memcpy( (char*)_s, (char*)s, length );
        }
    }
}

SimpleString::SimpleString()
{
    _init( NULL, 10 );
}

SimpleString::SimpleString( unsigned int nStartLength )
{
    _init( NULL, nStartLength );
}

SimpleString::SimpleString( const unsigned char *s )
{
    const unsigned char *t = (s==NULL) ? (const unsigned char*)"" : (const unsigned char*)s;
    _init( t, strlen( (const char*)t ) );
}

SimpleString::SimpleString( const char *s )
{
    const unsigned char *t = (s==NULL) ? (const unsigned char*)"" : (const unsigned char*)s;
    _init( t, strlen( (const char*)t ) );
}

SimpleString::SimpleString( const unsigned char *s, unsigned int length )
{
    const unsigned char *t = (s==NULL) ? (const unsigned char*)"" : (const unsigned char*)s;
    _init( t, length );
}

SimpleString::SimpleString( const char *s, unsigned int length )
{
    const unsigned char *t = (s==NULL) ? (const unsigned char*)"" : (const unsigned char*)s;
    _init( t, length );
}

SimpleString::SimpleString( const SimpleString &s )
{
    _init( s.getPointer(), s.length() );
}

SimpleString::SimpleString( const SimpleString *s )
{
    _init( s->getPointer(), s->length() );
}

SimpleString::~SimpleString()
{
    delete _s;
}

//----

void SimpleString::_append( const char *s, int m )
{
    if( _n + m >= _nAllocated )
    {
        _nAllocated = _n + m + 10;
        unsigned char *p = new unsigned char[_nAllocated];
        memset( (char*)(p+_n), 0, _nAllocated - _n );
        if( _n != 0 )
        {
            memcpy( (char*)p, (char*)_s, _n );
        }
        delete _s;
        _s = p;
    }

    if( m > 0 )
    {
        memcpy( (char*)(_s+_n), (const char*)s, m );
        _n += m;
    }
}

SimpleString &SimpleString::append( unsigned char c )
{
    _append( (const char*)&c, 1 );
    return *this;
}

SimpleString &SimpleString::append( char c )
{
    _append( &c, 1 );
    return *this;
}

SimpleString &SimpleString::append( const unsigned char *s )
{
    unsigned int n = strlen( (const char*)s );
    _append( (const char*)s, n );
    return *this;
}

SimpleString &SimpleString::append( const char *s )
{
    unsigned int n = strlen( s );
    _append( s, n );
    return *this;
}

SimpleString &SimpleString::append( const SimpleString &s )
{
    _append( s.getPointerC(), s.length() );
    return *this;
}

SimpleString &SimpleString::append( const SimpleString *s )
{
    _append( s->getPointerC(), s->length() );
    return *this;
}

SimpleString &SimpleString::empty( void )
{
    _n = 0;
    memset( (char*)_s, 0, _nAllocated );
    return *this;
}

SimpleString &SimpleString::assign( const unsigned char *s )
{
    empty();
    return append( s );
}

SimpleString &SimpleString::assign( const char *s )
{
    empty();
    return append( s );
}

SimpleString &SimpleString::assign( const SimpleString &s )
{
    empty();
    return append( s );
}

SimpleString &SimpleString::assign( const SimpleString *s )
{
    empty();
    return append( s );
}

//----

SimpleString &SimpleString::chopWhiteSpace( void )
{
    chopWhiteSpaceRight();
    chopWhiteSpaceLeft();
    return *this;
}

SimpleString &SimpleString::chopWhiteSpaceLeft( void )
{
    unsigned int i = 0;
    while( i < _n )
    {
        if( _s[i] != (unsigned char)' ' &&
            _s[i] != (unsigned char)'\t' )
        {
            break;
        }
        i++;
    }

    int j = 0;
    while( i < _nAllocated )
    {
        _s[j++] = _s[i++];
    }
    return *this;
}

SimpleString &SimpleString::chopWhiteSpaceRight( void )
{
    unsigned int i = _n;
    while( i > 0 )
    {
        i--;
        if( _s[i] != (unsigned char)' ' &&
            _s[i] != (unsigned char)'\t' )
        {
            break;
        }
        _s[i] = '\0';
        _n--;
    }
    return *this;
}

//----

bool SimpleString::isEmpty( void ) const
{
    return (_n == 0);
}

bool SimpleString::equals( const unsigned char *s ) const
{
    return equals( (const char*)s );
}

bool SimpleString::equals( const char *s ) const
{
    unsigned int m = strlen(s);
    if( _n != m )
    {
        return false;
    }
    if( _n != 0 && memcmp( (char*)_s, s, _n ) != 0 )
    {
        return false;
    }
    return true;
}

bool SimpleString::equals( const SimpleString &s ) const
{
    if( _n != s.length() )
    {
        return false;
    }
    if( _n != 0 && memcmp( (char*)_s, s.getPointerC(), _n ) != 0 )
    {
        return false;
    }
    return true;
}

bool SimpleString::equals( const SimpleString *s ) const
{
    return equals( *s );
}

SimpleString &SimpleString::left( unsigned int offset )
{
    return mid( 0, offset );
}

SimpleString &SimpleString::mid( unsigned int offsetLeft, unsigned int offsetRight )
{
    if( offsetRight > _n )
    {
        offsetRight = _n;
    }
    if( offsetLeft > offsetRight )
    {
        offsetLeft = offsetRight;
    }

    unsigned int i = 0;
    while( offsetLeft < offsetRight )
    {
        _s[i++] = _s[offsetLeft++];
    }
    _n = i;
    while( i < _nAllocated )
    {
        _s[i++] = (unsigned char)'\0';
    }
    return *this;
}

SimpleString &SimpleString::right( unsigned int offset )
{
    return mid( offset, _n );
}

int SimpleString::indexOf( unsigned char c ) const
{
    unsigned int i = 0;
    while( i < _n )
    {
        if( _s[i] == c )
        {
            return i;
        }
        i++;
    }
    return -1;
}

int SimpleString::indexOf( char c ) const
{
    return indexOf( (unsigned char)c );
}

unsigned int SimpleString::length( void ) const
{
    return _n;
}

unsigned char SimpleString::getAt( unsigned int offset ) const
{
    if( offset > _n )
    {
        return (unsigned char)'\0';
    }
    return _s[offset];
}

char SimpleString::getAtC( unsigned int offset ) const
{
    if( offset > _n )
    {
        return (char)'\0';
    }
    return (char)_s[offset];
}

const unsigned char *SimpleString::getPointer() const
{
    return _s;
}

const char *SimpleString::getPointerC() const
{
    return (const char*)_s;
}

SimpleString &SimpleString::clone( void ) const
{
    return *new SimpleString( _s, _n );
}

/* */
